/*
 * This file is part of the segmentation and registration toolkit (SphereSegRegTk).
 *
 * SphereSegRegTk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SphereSegRegTk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SphereSegRegTk.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CDICOMIMAGE_HH
#define CDICOMIMAGE_HH

#include <dcmtk/dcmimgle/dcmimage.h>

class CDicomImage
{
public:
	int width, height;		// width and height of dicom image slice
	float min_value, max_value;	// min and max values for the voxels
	float *data;


	/**
	 * Open DICOM image
	 * \param file	filename of dicom image
	 */
	CDicomImage(string &file)
	{
		data = NULL;

		DicomImage *image = new DicomImage(file.c_str());
		if (image == NULL)
		{
			error = true;
			std::cerr << "Error: cannot load DICOM image" << std::endl;
			return;
		}

		if (image->getStatus() != EIS_Normal)
		{
			std::cerr << "Error: cannot load DICOM image (" << DicomImage::getString(image->getStatus()) << ")" << std::endl;
			error = true;
			delete image;
			return;
		}
		if (!(image->isMonochrome()))
		{
			std::cerr << "Only monochrome images are supported so far" << std::endl;
			error = true;
			delete image;
			return;
		}

		// loa dwidth and heigh
		width = image->getWidth();
		height = image->getHeight();

		// allocate data array
		data = new float[width*height];

		// load min and max values of image
		double a, b;
		image->getMinMaxValues(a, b, 0);
		min_value = (float)a;
		max_value = (float)b;

		// get handler to pixel data
		const DiPixel *pix = image->getInterData();

		// copy dicom pixel data to storage array
		float *dst = data;
		short *src = (short*)pix->getData();
		for(int i = 0; i < width*height; i++)
		{
			*dst = *src;
			dst++;
			src++;
		}
		error = false;
		delete image;
	}


	/**
	 * Deconstructor
	 */
	~CDicomImage()
	{
		if (data)
			delete data;
	}

	/**
	 * fast error access operator!
	 */
	bool operator!()
	{
		return error;
	}


private:
	bool error;
};

#endif
