/*
 * This file is part of the segmentation and registration toolkit (SphereSegRegTk).
 *
 * SphereSegRegTk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SphereSegRegTk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SphereSegRegTk.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CWINDOWMAIN_HPP
#define CWINDOWMAIN_HPP

#include <libglademm.h>
#include <libglademm/xml.h>
#include <gtkmm.h>
#include <iostream>
#include <cairomm/context.h>
#include "CFilterHandler.hpp"
#include "libseg/CPointRadius.hh"


template <typename T>
class CWindowMain :	public
			Gtk::Window
{
public:

	CWindowMain(BaseObjectType* p_cobject, const Glib::RefPtr<Gnome::Glade::Xml>& p_refGlade);
	~CWindowMain();

private:
	// interface to filters
	CFilterHandler<T> filterHandler;

	// reference to glade xml interface to get widgets
	Glib::RefPtr<Gnome::Glade::Xml> refGlade;

	// reference to cairo
	Gtk::DrawingArea *drawingarea;
	Cairo::RefPtr<Cairo::Context> cairo_context;
	Gtk::HScale *hscale_image;
	void on_hscale_image_signal_value_changed();
	int selected_3d_slice;

	// statusbar on bottom of window
	Gtk::Statusbar *statusbar;

	// notebook for filters
	Gtk::Notebook *notebook_filters;
	void on_notebook_switch_page(GtkNotebookPage* page, guint page_num);

	// raw buttons
	Gtk::FileChooserDialog *raw_dialog;
	Gtk::FileChooserDialog *reference_dialog;
	Gtk::FileChooserDialog *matching_dialog;
	void on_button_raw_open_clicked();
	void on_button_raw_sphere_apply();
	void on_button_copy_raw_window_to_sphere_parameters_clicked();
	Gtk::Label* label_raw_current_value;
	Gtk::Label* label_raw_window_min_value;
	Gtk::Label* label_raw_window_max_value;
	Gtk::SpinButton *spinbutton_raw_extra_window_padding;
	int raw_drag_start_x, raw_drag_start_y;	// start coordinates of dragging
	int raw_drag_pos_x, raw_drag_pos_y;	// coordinates of recent dragging position
	bool raw_drag_flag;
	float raw_window_min_value, raw_window_max_value;
	void on_button_raw_scale_to_half_clicked();

	// sphere buttons
	void on_button_sphere_apply();
	void on_button_sphere_gradient_apply();
	Gtk::SpinButton *spinbutton_sphere_min_value;
	Gtk::SpinButton *spinbutton_sphere_max_value;
	Gtk::SpinButton *spinbutton_sphere_min_radius;
	Gtk::SpinButton *spinbutton_sphere_valid_error;
	Gtk::SpinButton *spinbutton_sphere_fft_threads;
	void on_button_sphere_preview();

	// gradient buttons
	void on_button_gradient_apply();
	void on_button_gradient_particles_apply();

	// particle buttons
	void on_button_particles_apply();
	void on_button_particles_graph_apply();
	Gtk::SpinButton *spinbutton_particles_init_distance;
	Gtk::SpinButton *spinbutton_particles_stop_gradient;
	Gtk::SpinButton *spinbutton_particles_start_steps;
	Gtk::SpinButton *spinbutton_particles_min_start_radius;

	// graph buttons
	void on_button_graph_apply();
	void on_button_graph_matching_apply();
	Gtk::SpinButton *spinbutton_graph_min_next_distance;
	Gtk::SpinButton *spinbutton_graph_max_next_distance;
	Gtk::SpinButton *spinbutton_graph_ignore_start_border_size;
	Gtk::SpinButton *spinbutton_graph_connect_strip_distance_scale;
	Gtk::Entry *entry_graph_export_filename;
	void on_button_graph_export();

	// matching buttons
	void on_button_matching_apply();

	// simple matching
	void on_button_matching_choose_graph_file();
	Gtk::Entry *entry_reference_graph_file;
	Gtk::Label *label_reference_graph_valid;
	void on_button_matching_load_graph();

	Gtk::SpinButton *spinbutton_matching_simple_unprojected_distance;
	Gtk::SpinButton *spinbutton_matching_simple_min_matrix_generation_point_distance;
	Gtk::CheckButton *checkbutton_matching_mutual_graph_distance;
	Gtk::SpinButton *spinbutton_matching_simple_adaptive_max_points;
	Gtk::SpinButton *spinbutton_matching_simple_adaptive_start_min_distance;
	Gtk::SpinButton *spinbutton_matching_simple_adaptive_start_min_radius;
	Gtk::SpinButton *spinbutton_matching_max_connection_distance;
	Gtk::CheckButton *checkbutton_matching_simple_use_point_to_line_distance;

	Gtk::SpinButton *spinbutton_matching_simple_max_rotation_angle;
	Gtk::SpinButton *spinbutton_matching_simple_min_scale_factor;
	Gtk::SpinButton *spinbutton_matching_simple_max_scale_factor;
	Gtk::SpinButton *spinbutton_matching_simple_min_shear_factor;
	Gtk::SpinButton *spinbutton_matching_simple_max_shear_factor;
	Gtk::SpinButton *spinbutton_matching_simple_max_translation_dist_x;
	Gtk::SpinButton *spinbutton_matching_simple_max_translation_dist_y;
	Gtk::SpinButton *spinbutton_matching_simple_max_translation_dist_z;
	Gtk::SpinButton *spinbutton_matching_simple_pre_translation_x;
	Gtk::SpinButton *spinbutton_matching_simple_pre_translation_y;
	Gtk::SpinButton *spinbutton_matching_simple_pre_translation_z;

	Gtk::Label *label_filter_matching_transformation_matrix1;
	Gtk::Label *label_filter_matching_transformation_matrix2;
	void on_button_copy_matrix_to_transformation_filter();

	// transformation filter
	Gtk::SpinButton *spinbutton_matching_matrix[4][4];
	void on_button_transformation_apply();
	void on_button_transformation_load_identity_matrix();

	void on_button_transformation_choose_reference_data_file();
	Gtk::Entry *entry_transformation_reference_data_file;
	void on_button_transformation_load_reference_data();

	Gtk::RadioButton *radiobutton_transformation_display1;
	Gtk::RadioButton *radiobutton_transformation_display2;
	Gtk::RadioButton *radiobutton_transformation_display3;
	void on_radiobutton_transformation_display_toggled();
	void on_spinbutton_matching_max_connection_distance();

	void on_button_transformation_raw_scale_to_half_clicked();


	// toolbuttons
	void on_toolbutton_save();
	void on_toolbutton_open();
	Gtk::CheckButton *checkbutton_save_with_parameters;
	Gtk::CheckButton *checkbutton_open_with_parameters;

	bool on_drawingarea_motion_notify_event(GdkEventMotion *event);
	bool on_drawingarea_button_press_event(GdkEventButton *event);
	bool on_drawingarea_button_release_event(GdkEventButton *event);
	bool on_drawingarea_expose_event(GdkEventExpose *event);
	void recompute_raw_circle_values();

	void update_hscale_limits();

	// helper function for standard 'open' and 'open with loading extra data'
	bool load_raw_data_with_dialog();


	void set_gui_parameter(	Gtk::SpinButton	*spin_button,
				typename CFilterHandler<T>::SParam *params,
				const char *button_name,
				string &missing_params
			);
	void set_gui_parameter(	Gtk::CheckButton	*check_button,
				typename CFilterHandler<T>::SParam *params,
				const char *button_name,
				string &missing_params
			);
	void set_gui_parameter(	Gtk::Entry	*entry,
				typename CFilterHandler<T>::SParam *params,
				const char *button_name,
				string &missing_params
			);
	/**
	 * draw the graph with the color specified by color[3]
	 * if color[0] equals to -1, the colors are rotated for every edge
	 */
	void draw_graph(CGraph<3,T> &graph, float color[4], Cairo::RefPtr<Cairo::Context>& cr, float disp_x, float disp_y, CMatrix4<T> *matrix);
};




/**
 * initialize main window
 *
 * connect signals, load widget handlers, initialize dialogs and classes
 */
template<typename T>
CWindowMain<T>::CWindowMain(BaseObjectType *p_cobject, const Glib::RefPtr<Gnome::Glade::Xml>& p_refGlade)
	:
	Gtk::Window(p_cobject),
	refGlade(p_refGlade)
{
	// connect signals

	// BUTTONS
	Gtk::Button *b;

	// RAW PAGE
	refGlade->get_widget("button_raw_open", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_raw_open_clicked	));
	refGlade->get_widget("spinbutton_raw_extra_window_padding",	spinbutton_raw_extra_window_padding);
	refGlade->get_widget("button_raw_sphere_apply", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_raw_sphere_apply	));
	refGlade->get_widget("button_copy_raw_window_to_sphere_parameters", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_copy_raw_window_to_sphere_parameters_clicked	));
	refGlade->get_widget("label_raw_current_value", label_raw_current_value);
	refGlade->get_widget("label_raw_window_min_value", label_raw_window_min_value);
	refGlade->get_widget("label_raw_window_max_value", label_raw_window_max_value);
	raw_drag_start_x = -1;		raw_drag_start_y = -1;
	raw_window_min_value = 0;	raw_window_max_value = 0;
	raw_drag_flag = false;
	refGlade->get_widget("button_raw_scale_to_half", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_raw_scale_to_half_clicked	));

	// create choose file dialog
	raw_dialog = new Gtk::FileChooserDialog("Please choose an image file", Gtk::FILE_CHOOSER_ACTION_OPEN);
	raw_dialog->set_transient_for(*this);
	raw_dialog->add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	raw_dialog->add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

	Gtk::FileFilter raw_filter_image;
	raw_filter_image.set_name("DICOM files");
//	raw_filter_image.add_mime_type("image/*");
	raw_filter_image.add_pattern("*.IMA");
	raw_filter_image.add_pattern("*.ima");
	raw_filter_image.add_pattern("*.dcm");
	raw_dialog->add_filter(raw_filter_image);

	Gtk::FileFilter raw_filter_any;
	raw_filter_any.set_name("Any file");
	raw_filter_any.add_pattern("*");
	raw_dialog->add_filter(raw_filter_any);


	// SPHERE PAGE
	refGlade->get_widget("button_sphere_apply", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_sphere_apply	));
	refGlade->get_widget("button_sphere_gradient_apply", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_sphere_gradient_apply	));
	refGlade->get_widget("spinbutton_sphere_min_value",	spinbutton_sphere_min_value);
	refGlade->get_widget("spinbutton_sphere_min_radius",	spinbutton_sphere_min_radius);
	refGlade->get_widget("spinbutton_sphere_valid_error",	spinbutton_sphere_valid_error);
	refGlade->get_widget("spinbutton_sphere_fft_threads",	spinbutton_sphere_fft_threads);
	refGlade->get_widget("spinbutton_sphere_max_value",	spinbutton_sphere_max_value);
	refGlade->get_widget("button_sphere_preview", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_sphere_preview	));

	// GRADIENT PAGE
	refGlade->get_widget("button_gradient_apply", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_gradient_apply	));
	refGlade->get_widget("button_gradient_particles_apply", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_gradient_particles_apply	));

	// PARTICLES PAGE
	refGlade->get_widget("button_particles_apply", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_particles_apply	));
	refGlade->get_widget("button_particles_graph_apply", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_particles_graph_apply	));
	refGlade->get_widget("spinbutton_particles_init_distance",	spinbutton_particles_init_distance);
	refGlade->get_widget("spinbutton_particles_stop_gradient",	spinbutton_particles_stop_gradient);
	refGlade->get_widget("spinbutton_particles_start_steps",	spinbutton_particles_start_steps);
	refGlade->get_widget("spinbutton_particles_min_start_radius",	spinbutton_particles_min_start_radius);

	// GRAPH PAGE
	refGlade->get_widget("button_graph_apply", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_graph_apply	));
	refGlade->get_widget("button_graph_matching_apply", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_graph_matching_apply	));
	refGlade->get_widget("spinbutton_graph_min_next_distance",	spinbutton_graph_min_next_distance);
	refGlade->get_widget("spinbutton_graph_max_next_distance",	spinbutton_graph_max_next_distance);
	refGlade->get_widget("spinbutton_graph_ignore_start_border_size",	spinbutton_graph_ignore_start_border_size);
	refGlade->get_widget("spinbutton_graph_connect_strip_distance_scale",	spinbutton_graph_connect_strip_distance_scale);
	refGlade->get_widget("entry_graph_export_filename",	entry_graph_export_filename);
	refGlade->get_widget("button_graph_export", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_graph_export	));

	// MATCHING PAGE
	refGlade->get_widget("button_matching_apply", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_matching_apply	));
	refGlade->get_widget("button_matching_choose_graph_file", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_matching_choose_graph_file	));
	refGlade->get_widget("button_matching_load_graph", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_matching_load_graph	));
	refGlade->get_widget("button_copy_matrix_to_transformation_filter", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_copy_matrix_to_transformation_filter	));

	refGlade->get_widget("entry_reference_graph_file",	entry_reference_graph_file);
	refGlade->get_widget("label_reference_graph_valid",	label_reference_graph_valid);

	refGlade->get_widget("spinbutton_matching_simple_unprojected_distance",	spinbutton_matching_simple_unprojected_distance);
	refGlade->get_widget("spinbutton_matching_simple_min_matrix_generation_point_distance",	spinbutton_matching_simple_min_matrix_generation_point_distance);
	refGlade->get_widget("checkbutton_matching_mutual_graph_distance",	checkbutton_matching_mutual_graph_distance);
	refGlade->get_widget("spinbutton_matching_simple_adaptive_max_points",	spinbutton_matching_simple_adaptive_max_points);
	refGlade->get_widget("spinbutton_matching_simple_adaptive_start_min_distance",	spinbutton_matching_simple_adaptive_start_min_distance);
	refGlade->get_widget("spinbutton_matching_simple_adaptive_start_min_radius",	spinbutton_matching_simple_adaptive_start_min_radius);
	refGlade->get_widget("checkbutton_matching_simple_use_point_to_line_distance",	checkbutton_matching_simple_use_point_to_line_distance);

	refGlade->get_widget("spinbutton_matching_simple_max_rotation_angle",	spinbutton_matching_simple_max_rotation_angle);
	refGlade->get_widget("spinbutton_matching_simple_min_scale_factor",	spinbutton_matching_simple_min_scale_factor);
	refGlade->get_widget("spinbutton_matching_simple_max_scale_factor",	spinbutton_matching_simple_max_scale_factor);
	refGlade->get_widget("spinbutton_matching_simple_min_shear_factor",	spinbutton_matching_simple_min_shear_factor);
	refGlade->get_widget("spinbutton_matching_simple_max_shear_factor",	spinbutton_matching_simple_max_shear_factor);
	refGlade->get_widget("spinbutton_matching_simple_max_translation_dist_x",	spinbutton_matching_simple_max_translation_dist_x);
	refGlade->get_widget("spinbutton_matching_simple_max_translation_dist_y",	spinbutton_matching_simple_max_translation_dist_y);
	refGlade->get_widget("spinbutton_matching_simple_max_translation_dist_z",	spinbutton_matching_simple_max_translation_dist_z);
	refGlade->get_widget("spinbutton_matching_simple_pre_translation_x",	spinbutton_matching_simple_pre_translation_x);
	refGlade->get_widget("spinbutton_matching_simple_pre_translation_y",	spinbutton_matching_simple_pre_translation_y);
	refGlade->get_widget("spinbutton_matching_simple_pre_translation_z",	spinbutton_matching_simple_pre_translation_z);

	refGlade->get_widget("label_filter_matching_transformation_matrix1",	label_filter_matching_transformation_matrix1);
	refGlade->get_widget("label_filter_matching_transformation_matrix2",	label_filter_matching_transformation_matrix2);

	refGlade->get_widget("spinbutton_matching_max_connection_distance",	spinbutton_matching_max_connection_distance);
	if (spinbutton_matching_max_connection_distance)	spinbutton_matching_max_connection_distance->signal_changed().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_spinbutton_matching_max_connection_distance	));

	// create choose file dialog to select graph to match with
	matching_dialog = new Gtk::FileChooserDialog("Please choose an image file", Gtk::FILE_CHOOSER_ACTION_OPEN);
	matching_dialog->set_transient_for(*this);
	matching_dialog->add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	matching_dialog->add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

	Gtk::FileFilter matching_filter_image;
	matching_filter_image.set_name("Graph files");
	matching_filter_image.add_pattern("*_graph_raw.sav");
	matching_dialog->add_filter(matching_filter_image);

	Gtk::FileFilter matching_filter_any;
	matching_filter_any.set_name("Any file");
	matching_filter_any.add_pattern("*");
	matching_dialog->add_filter(matching_filter_any);


	// PROJECTION PAGE
	refGlade->get_widget("spinbutton_matching_matrix_00", spinbutton_matching_matrix[0][0]);
	refGlade->get_widget("spinbutton_matching_matrix_01", spinbutton_matching_matrix[0][1]);
	refGlade->get_widget("spinbutton_matching_matrix_02", spinbutton_matching_matrix[0][2]);
	refGlade->get_widget("spinbutton_matching_matrix_03", spinbutton_matching_matrix[0][3]);

	refGlade->get_widget("spinbutton_matching_matrix_10", spinbutton_matching_matrix[1][0]);
	refGlade->get_widget("spinbutton_matching_matrix_11", spinbutton_matching_matrix[1][1]);
	refGlade->get_widget("spinbutton_matching_matrix_12", spinbutton_matching_matrix[1][2]);
	refGlade->get_widget("spinbutton_matching_matrix_13", spinbutton_matching_matrix[1][3]);

	refGlade->get_widget("spinbutton_matching_matrix_20", spinbutton_matching_matrix[2][0]);
	refGlade->get_widget("spinbutton_matching_matrix_21", spinbutton_matching_matrix[2][1]);
	refGlade->get_widget("spinbutton_matching_matrix_22", spinbutton_matching_matrix[2][2]);
	refGlade->get_widget("spinbutton_matching_matrix_23", spinbutton_matching_matrix[2][3]);

	refGlade->get_widget("spinbutton_matching_matrix_30", spinbutton_matching_matrix[3][0]);
	refGlade->get_widget("spinbutton_matching_matrix_31", spinbutton_matching_matrix[3][1]);
	refGlade->get_widget("spinbutton_matching_matrix_32", spinbutton_matching_matrix[3][2]);
	refGlade->get_widget("spinbutton_matching_matrix_33", spinbutton_matching_matrix[3][3]);

	refGlade->get_widget("button_transformation_apply", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_transformation_apply	));
	refGlade->get_widget("button_transformation_load_identity_matrix", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_transformation_load_identity_matrix	));

	refGlade->get_widget("radiobutton_transformation_display1", radiobutton_transformation_display1);
	refGlade->get_widget("radiobutton_transformation_display2", radiobutton_transformation_display2);
	refGlade->get_widget("radiobutton_transformation_display3", radiobutton_transformation_display3);

	refGlade->get_widget("button_transformation_choose_reference_data_file", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_transformation_choose_reference_data_file	));

	refGlade->get_widget("entry_transformation_reference_data_file", entry_transformation_reference_data_file);

	refGlade->get_widget("button_transformation_load_reference_data", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_transformation_load_reference_data	)); 

	// create choose file dialog
	reference_dialog = new Gtk::FileChooserDialog("Please choose an image file", Gtk::FILE_CHOOSER_ACTION_OPEN);
	reference_dialog->set_transient_for(*this);
	reference_dialog->add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	reference_dialog->add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

	refGlade->get_widget("button_transformation_raw_scale_to_half", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_button_transformation_raw_scale_to_half_clicked	));

	refGlade->get_widget("radiobutton_transformation_display1", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_radiobutton_transformation_display_toggled	)); 
	refGlade->get_widget("radiobutton_transformation_display2", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_radiobutton_transformation_display_toggled	)); 
	refGlade->get_widget("radiobutton_transformation_display3", b);
	if (b)	b->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_radiobutton_transformation_display_toggled	)); 



	// TOOLBUTTONS
	Gtk::ToolButton *tb;

	refGlade->get_widget("toolbutton_save", tb);
	if (tb)	tb->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_toolbutton_save	));
	refGlade->get_widget("toolbutton_open", tb);
	if (tb)	tb->signal_clicked().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_toolbutton_open	));
	refGlade->get_widget("checkbutton_save_with_parameters", checkbutton_save_with_parameters);
	refGlade->get_widget("checkbutton_open_with_parameters", checkbutton_open_with_parameters);

	// NOTEBOOK
	refGlade->get_widget("notebook_filters", notebook_filters);
	if (notebook_filters)  notebook_filters->signal_switch_page().connect(sigc::mem_fun(*this,    &CWindowMain<T>::on_notebook_switch_page   ));

	// STATUSBAR
	refGlade->get_widget("statusbar", statusbar);

	// DRAWINGAREA
	refGlade->get_widget("drawingarea", drawingarea);
	selected_3d_slice = 0;

	if (drawingarea)
	{
		// create expost event when drawing area should be redrawn
		drawingarea->signal_expose_event().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_drawingarea_expose_event		));

		drawingarea->signal_motion_notify_event().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_drawingarea_motion_notify_event	));
		drawingarea->signal_button_press_event().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_drawingarea_button_press_event		));
		drawingarea->signal_button_release_event().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_drawingarea_button_release_event	));
	}

	refGlade->get_widget("hscale_image", hscale_image);
	if (hscale_image)
		hscale_image->signal_value_changed().connect(sigc::mem_fun(*this,	&CWindowMain<T>::on_hscale_image_signal_value_changed));
}

template<typename T>
CWindowMain<T>::~CWindowMain()
{
	delete raw_dialog;
	delete reference_dialog;
	delete matching_dialog;
	hide();
}


template<typename T>
void CWindowMain<T>::on_spinbutton_matching_max_connection_distance()
{
	drawingarea->queue_draw();
}


/**
 */
template<typename T>
void CWindowMain<T>::on_hscale_image_signal_value_changed()
{
	selected_3d_slice = (int)hscale_image->get_value();
	drawingarea->queue_draw();
}


/**
 * update the limits for the hscale_image widget
 */
template<typename T>
void CWindowMain<T>::update_hscale_limits()
{
	switch(notebook_filters->get_current_page())
	{
		case 0:	// raw data
			if (filterHandler.raw_valid)
			{
				hscale_image->set_range((float)0,(float)filterHandler.raw_data.d);
			}
			else
			{
				hscale_image->set_range((float)0,(float)1);
			}
			break;

		case 1:	// sphere data
			if (filterHandler.sphere_valid)
			{
				hscale_image->set_range((float)0,(float)filterHandler.sphere_data.d);
			}
			else
			{
				hscale_image->set_range((float)0,(float)1);
			}
			break;

		case 2:	// display gradient data
		case 3:
		case 4:
		case 5:
			if (filterHandler.gradient_valid)
				hscale_image->set_range((float)0,(float)filterHandler.gradient_data.d);
			else
				hscale_image->set_range((float)0,(float)1);
			break;

		case 6:	// display transformation data
			if (radiobutton_transformation_display1->get_active())
			{
				if (filterHandler.transformation_valid)
					hscale_image->set_range((float)0,(float)filterHandler.transformation_data.d);
				else
					hscale_image->set_range((float)0,(float)1);
			}
		       	else if (radiobutton_transformation_display2->get_active())
			{
				if (filterHandler.reference_valid)
					hscale_image->set_range((float)0,(float)filterHandler.reference_data.d);
				else
					hscale_image->set_range((float)0,(float)1);
			}
			else
			{
				if (filterHandler.reference_valid && filterHandler.transformation_valid)
					hscale_image->set_range((float)0,(float)filterHandler.reference_data.d);
				else
					hscale_image->set_range((float)0,(float)1);
			}
			break;
	}
}

/*******************************************************
 * FILTER RAW
 *******************************************************/

/**
 * load a raw image by popping up an dialog
 * \return	true, if image was loaded
 */
template<typename T>
bool CWindowMain<T>::load_raw_data_with_dialog()
{
	int result = raw_dialog->run();
	raw_dialog->hide();

	switch(result)
	{
		case (Gtk::RESPONSE_OK):
		{
			// get filename from dialog if response is positive
			string filename = raw_dialog->get_filename();

			// try to open the file
			if (filterHandler.open_raw_data(filename))
			{
				if (selected_3d_slice < 0)
				{
					selected_3d_slice = 0;
				}
				else
				{
					if (selected_3d_slice >= filterHandler.raw_data.d)
						selected_3d_slice = filterHandler.raw_data.d;
				}
				update_hscale_limits();

				// display ok message
				statusbar->push(string() + "'" + filename + "' loaded");
				drawingarea->queue_draw();
				return true;
			}
			else
			{
				// error message
				statusbar->push(string() + "failed to load '" + filename + "': " + filterHandler.lastErrorMessage);
				return false;
			}
		}
	}

	return false;
}



/**
 * load an image file to raw input store
 */
template<typename T>
void CWindowMain<T>::on_button_raw_open_clicked()
{
	load_raw_data_with_dialog();
}

template<typename T>
void CWindowMain<T>::on_button_raw_scale_to_half_clicked()
{
	filterHandler.scale_raw_to_half();
	update_hscale_limits();
	drawingarea->queue_draw();
}

template<typename T>
void CWindowMain<T>::on_button_transformation_raw_scale_to_half_clicked()
{
	filterHandler.scale_reference_to_half();
	update_hscale_limits();
	drawingarea->queue_draw();
}

template<typename T>
void CWindowMain<T>::on_button_raw_sphere_apply()
{
	notebook_filters->set_current_page(1);
	on_button_sphere_apply();
}

template<typename T>
void CWindowMain<T>::on_button_copy_raw_window_to_sphere_parameters_clicked()
{
	// copy parameters to sphere dialog
	float window_padding = fabs(raw_window_min_value - raw_window_max_value) * spinbutton_raw_extra_window_padding->get_value();

	spinbutton_sphere_min_value->set_value(raw_window_min_value - window_padding);
	spinbutton_sphere_max_value->set_value(raw_window_max_value + window_padding);
}

/*******************************************************
 * FILTER SPHERE
 *******************************************************/

template<typename T>
void CWindowMain<T>::on_button_sphere_apply()
{
	if (filterHandler.raw_valid)
	{
		// create sphere filter and set maximum radius to half of image width
		filterHandler.sphere_filter.create_sphere_cache(filterHandler.raw_data.w/4);

		// set parameters for sphere filter
		filterHandler.sphere_filter.setParameters(
						spinbutton_sphere_min_value->get_value(),
						spinbutton_sphere_max_value->get_value(),
						(int)spinbutton_sphere_min_radius->get_value(),
						spinbutton_sphere_valid_error->get_value(),
						spinbutton_sphere_fft_threads->get_value()
				);

		// dont include following phrases to this area to get appropriate error message
	}

	if (!filterHandler.apply_sphere_filter())
	{
		statusbar->push(string() + "ERROR: " + filterHandler.lastErrorMessage);
		return;
	}

	statusbar->push(string() + "Spherical data created");

	update_hscale_limits();
	drawingarea->queue_draw();
}


template<typename T>
void CWindowMain<T>::on_button_sphere_preview()
{
	if (filterHandler.raw_valid)
	{
		// create sphere filter and set maximum radius to half of image width
		filterHandler.sphere_filter.create_sphere_cache(filterHandler.raw_data.w/4);

		// set parameters for sphere filter
		filterHandler.sphere_filter.setParameters(
						spinbutton_sphere_min_value->get_value(),
						spinbutton_sphere_max_value->get_value(),
						(int)spinbutton_sphere_min_radius->get_value(),
						spinbutton_sphere_valid_error->get_value(),
						spinbutton_sphere_fft_threads->get_value()
				);
	}

	if (!filterHandler.apply_sphere_preview())
	{
		statusbar->push(string() + "ERROR: " + filterHandler.lastErrorMessage);
		return;
	}

	statusbar->push(string() + "Spherical data created");

	update_hscale_limits();
	drawingarea->queue_draw();
}

template<typename T>
void CWindowMain<T>::on_button_sphere_gradient_apply()
{
	notebook_filters->set_current_page(2);
	on_button_gradient_apply();
}

/*******************************************************
 * FILTER GRADIENT
 *******************************************************/

template<typename T>
void CWindowMain<T>::on_button_gradient_apply()
{
	if (!filterHandler.apply_gradient_filter())
	{
		statusbar->push(string() + "ERROR: " + filterHandler.lastErrorMessage);
		return;
	}

	statusbar->push(string() + "Gradient data created");

	update_hscale_limits();
	drawingarea->queue_draw();
}

template<typename T>
void CWindowMain<T>::on_button_gradient_particles_apply()
{
	notebook_filters->set_current_page(3);
	on_button_particles_apply();
}

/*******************************************************
 * FILTER PARTICLES
 *******************************************************/

template<typename T>
void CWindowMain<T>::on_button_particles_apply()
{
	filterHandler.particles_filter.setParameters(
					(int)spinbutton_particles_init_distance->get_value(),
					spinbutton_particles_stop_gradient->get_value(),
					(int)spinbutton_particles_start_steps->get_value(),
					(int)spinbutton_particles_min_start_radius->get_value()
			);

	if (!filterHandler.apply_particles_filter())
	{
		statusbar->push(string() + "ERROR: " + filterHandler.lastErrorMessage);
		return;
	}

	statusbar->push(string() + "Particle data created");

	update_hscale_limits();
	drawingarea->queue_draw();
}

template<typename T>
void CWindowMain<T>::on_button_particles_graph_apply()
{
	notebook_filters->set_current_page(4);
	on_button_graph_apply();
}

/*******************************************************
 * FILTER GRAPH
 *******************************************************/

template<typename T>
void CWindowMain<T>::on_button_graph_apply()
{
	// first update the parameters for particle filter for strip connections
	filterHandler.particles_filter.setParameters(
					(int)spinbutton_particles_init_distance->get_value(),
					spinbutton_particles_stop_gradient->get_value(),
					(int)spinbutton_particles_start_steps->get_value(),
					(int)spinbutton_particles_min_start_radius->get_value()
			);

	filterHandler.graph_filter.setParameters(
					spinbutton_graph_min_next_distance->get_value(),
					spinbutton_graph_max_next_distance->get_value(),
					spinbutton_graph_ignore_start_border_size->get_value(),
					spinbutton_graph_connect_strip_distance_scale->get_value()
			);

	if (!filterHandler.apply_graph_filter())
	{
		statusbar->push(string() + "ERROR: " + filterHandler.lastErrorMessage);
		return;
	}

	statusbar->push(string() + "Graph created");

	update_hscale_limits();
	drawingarea->queue_draw();
}

template<typename T>
void CWindowMain<T>::on_button_graph_export()
{
	string filename = entry_graph_export_filename->get_text();
	filterHandler.graph_filter.export_polyline(filename);
}

/*******************************************************
 * FILTER MATCHING
 *******************************************************/

template<typename T>
void CWindowMain<T>::on_button_graph_matching_apply()
{
	notebook_filters->set_current_page(5);
	on_button_matching_apply();
}

template<typename T>
void CWindowMain<T>::on_button_matching_apply()
{
	filterHandler.matching_simple_filter.setParameters(
					spinbutton_matching_simple_unprojected_distance->get_value(),
					spinbutton_matching_simple_min_matrix_generation_point_distance->get_value(),
					checkbutton_matching_mutual_graph_distance->get_active(),
					spinbutton_matching_simple_adaptive_max_points->get_value(),
					spinbutton_matching_simple_adaptive_start_min_distance->get_value(),
					spinbutton_matching_simple_adaptive_start_min_radius->get_value(),
					checkbutton_matching_simple_use_point_to_line_distance->get_active(),

					spinbutton_matching_simple_max_rotation_angle->get_value()*(M_PI/180.0),
					spinbutton_matching_simple_min_scale_factor->get_value(),
					spinbutton_matching_simple_max_scale_factor->get_value(),
					spinbutton_matching_simple_min_shear_factor->get_value(),
					spinbutton_matching_simple_max_shear_factor->get_value(),
					spinbutton_matching_simple_max_translation_dist_x->get_value(),
					spinbutton_matching_simple_max_translation_dist_y->get_value(),
					spinbutton_matching_simple_max_translation_dist_z->get_value(),

					spinbutton_matching_simple_pre_translation_x->get_value(),
					spinbutton_matching_simple_pre_translation_y->get_value(),
					spinbutton_matching_simple_pre_translation_z->get_value()
			);

	if (!filterHandler.apply_matching_simple_filter())
	{
		statusbar->push(string() + "ERROR: " + filterHandler.lastErrorMessage);
		return;
	}

	statusbar->push("Simple matching created");

	CMatrix4<T> matrix_orig;
	CVector<3,T> rotation_angles;
	CVector<3,T> scaling;
	CVector<3,T> shearing;
	CVector<3,T> translation;

	filterHandler.matching_simple_filter.decompose_and_filter_matrix(	
					filterHandler.matching_simple_filter.best_matrix_trans,
					rotation_angles,
					scaling,
					shearing,
					translation
				);
	std::ostringstream buf1;
	std::ostringstream buf2;

	if (filterHandler.matching_simple_filter.min_distance != std::numeric_limits<T>::max())
	{
		buf1 << "<span font_family=\"fixed\"><small>";
		buf1 << std::setiosflags(ios::fixed) << std::setprecision(5);
		buf1 << filterHandler.matching_simple_filter.best_pre_translated_matrix_trans;
		buf1 << std::endl;
		buf1 << "rotation_angles: " << rotation_angles << std::endl;
		buf1 << "        scaling: " << scaling;
		buf1 << "</small></span>";

		buf2 << "<span font_family=\"fixed\"><small>";
		buf2 << std::setiosflags(ios::fixed) << std::setprecision(5);
		buf2 << filterHandler.matching_simple_filter.best_matrix_trans;
		buf2 << std::endl;
		buf2 << "   shearing: " << shearing << std::endl;
		buf2 << "translation: " << translation;
		buf2 << "</small></span>";
	}
	else
	{
		buf1 << "\n";
		buf1 << "<b>no matching found</b>";
		buf1 << "\n";
	}

	label_filter_matching_transformation_matrix1->set_markup(buf1.str());
	label_filter_matching_transformation_matrix2->set_markup(buf2.str());

	update_hscale_limits();
	drawingarea->queue_draw();

}

/**
 * choose a graph file to match with the computed graph
 */
template<typename T>
void CWindowMain<T>::on_button_matching_choose_graph_file()
{
	int result = matching_dialog->run();
	matching_dialog->hide();

	switch(result)
	{
		case (Gtk::RESPONSE_OK):
		{
			// get filename from dialog if response is positive
			entry_reference_graph_file->set_text(matching_dialog->get_filename());
		}
	}
}

/**
 * load the matching graph from file to program
 */
template<typename T>
void CWindowMain<T>::on_button_matching_load_graph()
{
	string filename = entry_reference_graph_file->get_text();

	// try to open the file
	if (filterHandler.load_graph_raw(filename))
	{
		// display ok message
		statusbar->push(string() + "'" + filename + "' loaded");
		drawingarea->queue_draw();
		label_reference_graph_valid->set_markup("<b>true</b>");
	}
	else
	{
		// error message
		statusbar->push(string() + "failed to load '" + filename + "'");
	}
}

/*******************************************************
 * FILTER PROJECTION
 *******************************************************/

template<typename T>
void CWindowMain<T>::on_button_copy_matrix_to_transformation_filter()
{
	for (int x = 0; x < 4; x++)
		for (int y = 0; y < 4; y++)
			spinbutton_matching_matrix[x][y]->set_value(filterHandler.matching_simple_filter.best_pre_translated_matrix_trans[x][y]);
}

template<typename T>
void CWindowMain<T>::on_button_transformation_load_identity_matrix()
{
	CMatrix4<float> m;
	m.loadIdentity();

	for (int x = 0; x < 4; x++)
		for (int y = 0; y < 4; y++)
			spinbutton_matching_matrix[x][y]->set_value(m[x][y]);
}

template <typename T>
void CWindowMain<T>::on_button_transformation_apply()
{
	CMatrix4<T> matrix;
	for (int x = 0; x < 4; x++)
		for (int y = 0; y < 4; y++)
			matrix[x][y] = spinbutton_matching_matrix[x][y]->get_value();

	filterHandler.transformation_filter.setParameters(matrix);

	if (!filterHandler.apply_transformation_filter())
	{
		statusbar->push(string() + "ERROR: " + filterHandler.lastErrorMessage);
		return;
	}

	update_hscale_limits();
	drawingarea->queue_draw();
	return;
}

/**
 * load a raw image by popping up an dialog
 * \return	true, if image was loaded
 */
template<typename T>
void CWindowMain<T>::on_button_transformation_load_reference_data()
{
	// get filename from dialog if response is positive
	string filename = entry_transformation_reference_data_file->get_text();

	// try to open the file
	if (filterHandler.open_reference_data(filename))
	{
		if (selected_3d_slice < 0)
		{
			selected_3d_slice = 0;
		}
		else
		{
			if (selected_3d_slice >= filterHandler.reference_data.d)
				selected_3d_slice = filterHandler.reference_data.d;
		}
		update_hscale_limits();

		// display ok message
		statusbar->push(string() + "'" + filename + "' loaded");
		drawingarea->queue_draw();
		return;
	}
	else
	{
		// error message
		statusbar->push(string() + "failed to load '" + filename + "': " + filterHandler.lastErrorMessage);
		return;
	}

	return;
}


template<typename T>
void CWindowMain<T>::on_button_transformation_choose_reference_data_file()
{
	int result = reference_dialog->run();
	reference_dialog->hide();

	switch(result)
	{
		case (Gtk::RESPONSE_OK):
		{
			// get filename from dialog if response is positive
			entry_transformation_reference_data_file->set_text(reference_dialog->get_filename());
		}
	}
}

template<typename T>
void CWindowMain<T>::on_radiobutton_transformation_display_toggled()
{
	update_hscale_limits();
	drawingarea->queue_draw();
}

/*******************************************************
 *******************************************************
 * DRWAING AREA STUFF
 *******************************************************
 *******************************************************
 */
template<typename T>
void CWindowMain<T>::on_notebook_switch_page(GtkNotebookPage* page, guint page_num)
{
	update_hscale_limits();
	drawingarea->queue_draw();
}



/**
 * mouse button was pressed in drawing area
 * if the page 0 for raw view is selected at the notebook, this handler
 * starts building up a rectangle to get the window min/max values for the
 * values within the rectangle
 */
template<typename T>
bool CWindowMain<T>::on_drawingarea_button_press_event(GdkEventButton *event)
{
	if (event->button != 1)
		return true;

	raw_drag_start_x = (int)event->x;
	raw_drag_start_y = (int)event->y;
	raw_drag_pos_x = raw_drag_start_x;
	raw_drag_pos_y = raw_drag_start_y;

	raw_drag_flag = true;
	return true;
}

template<typename T>
bool CWindowMain<T>::on_drawingarea_button_release_event(GdkEventButton *event)
{
	if (event->button != 1)
		return true;

	raw_drag_flag = false;

	return true;
}


template<typename T>
void CWindowMain<T>::recompute_raw_circle_values()
{
	if (raw_drag_start_x >= 0)
	{
		// if it's positive, the left mouse button is pressed and a rectangle should be created

		CVector<2,float> delta((raw_drag_start_x - raw_drag_pos_x), (float)(raw_drag_start_y - raw_drag_pos_y));
		float radius_f = delta.length();
		int radius_pow_2 = (int)(radius_f*radius_f);
		int radius = (int)radius_f;

		int min_x = raw_drag_start_x - radius;
		int min_y = raw_drag_start_y - radius;
		int min_z = selected_3d_slice - radius;

		int max_x = raw_drag_start_x + radius;
		int max_y = raw_drag_start_y + radius;
		int max_z = selected_3d_slice + radius;

		// beware the borders!!!
		if (min_x < 0)	min_x = 0;
		else		if (min_x >= filterHandler.raw_data.w)		min_x = filterHandler.raw_data.w;

		if (min_y < 0)	min_y = 0;
		else		if (min_y >= filterHandler.raw_data.h)		min_y = filterHandler.raw_data.h;

		if (min_z < 0)	min_z = 0;
		else		if (min_z >= filterHandler.raw_data.d)		min_z = filterHandler.raw_data.d;

		if (max_x < 0)	max_x = 0;
		else		if (max_x >= filterHandler.raw_data.w)		max_x = filterHandler.raw_data.w;

		if (max_y < 0)	max_y = 0;
		else		if (max_y >= filterHandler.raw_data.h)		max_y = filterHandler.raw_data.h;

		if (max_z < 0)	max_z = 0;
		else		if (max_z >= filterHandler.raw_data.d)		max_z = filterHandler.raw_data.d;

		float min_value = +100000;
		float max_value = -100000;

		int px, py, pz;
		for (pz = min_z; pz < max_z; pz++)
		{
			int dpz = pz-selected_3d_slice;
			int dpz2 = dpz*dpz;
			for (py = min_y; py < max_y; py++)
			{
				int dpy = py-raw_drag_start_y;
				int dpyz2 = dpy*dpy + dpz2;
				for (px = min_x; px < max_x; px++)
				{
					int dpx = px-raw_drag_start_x;
					if (dpx*dpx+dpyz2 > radius_pow_2)	continue;

					float value = filterHandler.raw_data.get(px, py, pz);

					if (value < min_value)
						min_value = value;
					if (value > max_value)
						max_value = value;
				}
			}
		}



		std::ostringstream buf_min;
		buf_min << "<b>" << min_value << "</b>";
		label_raw_window_min_value->set_markup(buf_min.str());
		raw_window_min_value = min_value;

		std::ostringstream buf_max;
		buf_max << "<b>" << max_value << "</b>";
		label_raw_window_max_value->set_markup(buf_max.str());
		raw_window_max_value = max_value;
	}
}

/**
 * handler for mouse motion in drawing area
 */
template<typename T>
bool CWindowMain<T>::on_drawingarea_motion_notify_event(GdkEventMotion *event)
{
	if (notebook_filters->get_current_page() != 0)	// just for raw data!
		return true;

	if (!filterHandler.raw_valid)
		return true;

	if (!raw_drag_flag)
		return true;

	// update position if we have the button pressed
	raw_drag_pos_x = (int)event->x;
	raw_drag_pos_y = (int)event->y;

	if (raw_drag_pos_x >= 0 && raw_drag_pos_y >= 0)
	{
		if (filterHandler.raw_data.w > raw_drag_pos_x && filterHandler.raw_data.h > raw_drag_pos_y)
		{
			float value = filterHandler.raw_data.get(raw_drag_pos_x, raw_drag_pos_y, selected_3d_slice);

			std::ostringstream buf;
			buf << "<b>" << value << "</b>";
			label_raw_current_value->set_markup(buf.str());
		}
	}

	recompute_raw_circle_values();

	drawingarea->queue_draw();

	return true;
}

/**
 * redraw the drawingarea widget if expost event occurs
 */
template<typename T>
bool CWindowMain<T>::on_drawingarea_expose_event(GdkEventExpose *event)
{
	// get drawable window
	Glib::RefPtr<Gdk::Window> window = drawingarea->get_window();

	if (!window)
		return true;

	int width = 0, height = 0;
	bool valid = false;
	bool postprocess = false;
	bool gradient = false;
	Glib::RefPtr<Gdk::Pixbuf> pixbuf;

	switch(notebook_filters->get_current_page())
	{
		case 0:	// raw image
			if (!filterHandler.raw_valid)
				break;

			width = filterHandler.raw_data.w;
			height = filterHandler.raw_data.h;

			pixbuf = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, false, 8, width, height);
			if (!pixbuf)
				break;

			filterHandler.raw_data.storeData24bppAndScaleSlice(pixbuf->get_pixels(), filterHandler.raw_limits[0], filterHandler.raw_limits[1], selected_3d_slice, pixbuf->get_rowstride());
			valid = true;

			if (raw_drag_start_x >= 0)
				postprocess = true;
			break;

		case 1:	// sphere image
			if (!filterHandler.sphere_valid)
				break;

			width = filterHandler.sphere_data.w;
			height = filterHandler.sphere_data.h;

			pixbuf = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, false, 8, width, height);
			if (!pixbuf)
				break;

			filterHandler.sphere_data.storeData24bppAndScaleSlice(pixbuf->get_pixels(), filterHandler.sphere_limits[0], filterHandler.sphere_limits[1], selected_3d_slice, pixbuf->get_rowstride());
			valid = true;
			break;


		case 2:	// gradient image
			if (filterHandler.gradient_valid)
				gradient = true;

			if (!filterHandler.gradient_valid)
				break;

			gradient = true;
			break;

		case 3:	// particles
			if (filterHandler.gradient_valid)
				gradient = true;

			if (filterHandler.particles_valid)
				postprocess = true;

			break;

		case 4:	// graph
			if (filterHandler.gradient_valid)
				gradient = true;

			if (filterHandler.graph_valid)
				postprocess = true;

			break;

		case 5: // matching
			// first just check for available graph data to draw at least one graph
			if (filterHandler.raw_valid)
			{
				width = filterHandler.raw_data.w;
				height = filterHandler.raw_data.h;

				pixbuf = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, false, 8, width, height);
				if (!pixbuf)
					break;

				filterHandler.raw_data.storeData24bppAndScaleSlice(pixbuf->get_pixels(), filterHandler.raw_limits[0], filterHandler.raw_limits[1], selected_3d_slice, pixbuf->get_rowstride());
				valid = true;
			}
			else
			{
				if (filterHandler.gradient_valid)
					gradient = true;
			}
			if (filterHandler.graph_valid || filterHandler.reference_graph_valid)
				postprocess = true;
			break;

		case 6:	// transformation filter
			if (radiobutton_transformation_display1->get_active())
			{
				if (!filterHandler.transformation_valid)
					break;

				width = filterHandler.transformation_data.w;
				height = filterHandler.transformation_data.h;

				pixbuf = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, false, 8, width, height);
				if (!pixbuf)
					break;

				filterHandler.transformation_data.storeData24bppAndScaleSlice(pixbuf->get_pixels(), filterHandler.transformation_limits[0], filterHandler.transformation_limits[1], selected_3d_slice, pixbuf->get_rowstride());
				valid = true;
			}
		       	else if (radiobutton_transformation_display2->get_active())
			{
				if (!filterHandler.reference_valid)
					break;

				width = filterHandler.reference_data.w;
				height = filterHandler.reference_data.h;

				pixbuf = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, false, 8, width, height);
				if (!pixbuf)
					break;

				filterHandler.reference_data.storeData24bppAndScaleSlice(pixbuf->get_pixels(), filterHandler.reference_limits[0], filterHandler.reference_limits[1], selected_3d_slice, pixbuf->get_rowstride());
				valid = true;
			}
			else
			{
				// draw reference and destination ct data

				if (!filterHandler.transformation_valid || !filterHandler.reference_valid)
					break;

				if (	(filterHandler.reference_data.w != filterHandler.transformation_data.w)	||
					(filterHandler.reference_data.h != filterHandler.transformation_data.h)
				   )
				{
					std::cerr << "sizes of matching domains dont match" << std::endl;
					break;
				}

				width = filterHandler.reference_data.w*2;
				height = filterHandler.reference_data.h;

				pixbuf = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, false, 8, width, height);
				if (!pixbuf)
					break;

				T inv_raw_dist = (T)255.0f/(T)(filterHandler.reference_limits[1] - filterHandler.reference_limits[0]);
				T inv_proj_dist = (T)255.0f/(T)(filterHandler.transformation_limits[1] - filterHandler.transformation_limits[0]);

				unsigned char *pixels = pixbuf->get_pixels();
				for (int y = 0; y < filterHandler.transformation_data.h; y++)
				{
					for (int x = 0; x < filterHandler.transformation_data.w; x++)
					{
						T v1 =	(filterHandler.transformation_data.get(x,y,selected_3d_slice) - filterHandler.reference_limits[0])*inv_raw_dist;
						T v2 =	(filterHandler.reference_data.get(x,y,selected_3d_slice) - filterHandler.transformation_limits[0])*inv_proj_dist;

						int pos1 = (x+y*filterHandler.transformation_data.w*2+filterHandler.reference_data.w)*3;
						pixels[pos1+0] = (unsigned char)v1;
						pixels[pos1+1] = (unsigned char)v1;
						pixels[pos1+2] = (unsigned char)v1;

						int pos2 = (x+y*filterHandler.transformation_data.w*2)*3;
						pixels[pos2+0] = (unsigned char)v2;
						pixels[pos2+1] = (unsigned char)v2;
						pixels[pos2+2] = (unsigned char)v2;
					}
				}

				valid = true;
				postprocess = true;
			}
			break;
	}

	if (gradient)
	{
		width = filterHandler.gradient_data.w;
		height = filterHandler.gradient_data.h;

		pixbuf = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, false, 8, width, height);

		if (pixbuf)
		{
			filterHandler.gradient_data.storeData24bppAndScaleSlice(	
					pixbuf->get_pixels(),
				       	filterHandler.gradient_limits[0], 
					filterHandler.gradient_limits[1],
					selected_3d_slice,
					pixbuf->get_rowstride()
				);
			valid = true;
		}
	}

	if (valid)
	{
		drawingarea->set_size_request(width, height);

		pixbuf->render_to_drawable(window, drawingarea->get_style()->get_black_gc(),
				0, 0, 0, 0,			// src_x, src_y, dest_x, dest_y
				width, height,			// image size
				Gdk::RGB_DITHER_NONE, 0, 0);	// dither, x_dither, y_dither
	}

	if (!postprocess)
		return true;


	// postprocessing is required (drawing particles, graphs, etc.)

	// create and initialize cairo
	Cairo::RefPtr<Cairo::Context> cr = window->create_cairo_context();
	cr->rectangle(event->area.x, event->area.y, event->area.width, event->area.height);
	cr->clip();

	switch(notebook_filters->get_current_page())
	{
		case 0:
		{
			cr->set_source_rgb(1, 0, 0);
			cr->set_line_width(1.0);

			CVector<2,float> delta((raw_drag_start_x - raw_drag_pos_x), (float)(raw_drag_start_y - raw_drag_pos_y));
			float radius = delta.length();
			cr->arc(	raw_drag_start_x, raw_drag_start_y,	// center
					radius,		// radius
					0, 2*M_PI	// start + end angle
				);
			cr->stroke();

			recompute_raw_circle_values();
		}
			break;

		case 3:	// particles
		{
			cr->set_source_rgb(1, 1, 1);
			cr->set_line_width(1.0);

			typename list< CPointRadius<3,T> >::iterator iter = filterHandler.particles_filter.particleList.begin();
			for(; iter != filterHandler.particles_filter.particleList.end(); iter++)
			{
				CPointRadius<3,T> &p = *iter;
				if ((int)p.pos[2] != selected_3d_slice)
					continue;

				cr->move_to(p.pos[0], p.pos[1]);
				cr->line_to(p.pos[0]+1, p.pos[1]+1);
				cr->stroke();
			}
		}
			break;

		case 4:	// graph
		{
			float color[4] = {-1, -1, -1, -1};
			draw_graph(filterHandler.graph_filter.graph, color, cr);
		}
			break;

		case 5:	// matching
		{
			if (filterHandler.graph_valid)
			{
				float color[4] = {0, 1, 0, 1};
				draw_graph(filterHandler.graph_filter.graph, color, cr);
			}

			if (filterHandler.reference_graph_valid)
			{
				float color[4] = {0, 0, 1, 1};
				draw_graph(filterHandler.reference_graph, color, cr);
			}
		}
			break;	// case 5

		case 6:	// transformation
		{
			CMatrix4<T> matrix2;
			for (int x = 0; x < 4; x++)
				for (int y = 0; y < 4; y++)
					matrix2[x][y] = spinbutton_matching_matrix[x][y]->get_value();

			CMatrix4<T> matrix;
			if (!(matrix2.compute_inverse(matrix)))
			{
				std::cerr << "unable to compute inverse matrix" << std::endl;
				break;
			}

			float color1[4] = {0, 0, 1, 1};
			float color2[4] = {0, 1, 0, 1};
			float color3[4] = {0, 1, 0, 0.5};

			// draw graph
			if (filterHandler.reference_graph_valid)
			{
				draw_graph(filterHandler.reference_graph, color2, cr, 0, 0, NULL);
				draw_graph(filterHandler.reference_graph, color3, cr, filterHandler.transformation_data.w, 0, NULL);
			}
			if (filterHandler.graph_valid)
				draw_graph(filterHandler.graph_filter.graph, color1, cr, filterHandler.transformation_data.w, 0, &matrix);


			T max_distance = spinbutton_matching_max_connection_distance->get_value();;
			T max_distance2 = max_distance * max_distance;

			for (	typename list<typename CGraph<3,T>::CEdge>::iterator edges_i = filterHandler.reference_graph.edges.begin();
				edges_i != filterHandler.reference_graph.edges.end();
				edges_i++)
			{
				typename CGraph<3,T>::CEdge &edge = *edges_i;
				for (typename list<typename CGraph<3,T>::CNode>::iterator nodes_i = edge.nodes.begin(); nodes_i != edge.nodes.end(); nodes_i++)
				{
					typename CGraph<3,T>::CNode &node = *nodes_i;

					// projected node
					CVector<3,T> proj_node_pos = matrix2*node.pos;

					T min_distance2 = std::numeric_limits<T>::max();
					CVector<3,T> min_node2;		// destination node with minimum distance

					// search for node in destination graph with minimum distance
					for (	typename list<typename CGraph<3,T>::CEdge>::iterator edges2_i = filterHandler.graph_filter.graph.edges.begin();
						edges2_i != filterHandler.graph_filter.graph.edges.end();
						edges2_i++)
					{
						typename CGraph<3,T>::CEdge &edge2 = *edges2_i;
						for (typename list<typename CGraph<3,T>::CNode>::iterator nodes2_i = edge2.nodes.begin(); nodes2_i != edge2.nodes.end(); nodes2_i++)
						{
							CVector<3,T> node2_pos = (*nodes2_i).pos;
							T distance2 = proj_node_pos.dist2(node2_pos);
							if (distance2 < min_distance2)
							{
								min_node2 = node2_pos;
								min_distance2 = distance2;
							}
						}
					}

					// destination node projected to screen
					CVector<3,T> proj_min_node2 = matrix*min_node2;

					// TODO: draw graph node with minimum distance

					float fade_in_distance = 10;
					float inv_fade_in_distance = 1.0 / fade_in_distance;

					float low_z, high_z;
					if (node.pos[2] < proj_min_node2[2])
					{
						low_z = node.pos[2];
						high_z = proj_min_node2[2];
					}
					else
					{
						high_z = node.pos[2];
						low_z = proj_min_node2[2];
					}

					// out of fade distance
					if (low_z - fade_in_distance > selected_3d_slice)
						continue;

					if (high_z + fade_in_distance < selected_3d_slice)
						continue;

					float alpha;
					if (low_z > selected_3d_slice)
						alpha = 1.0 - (low_z - (float)selected_3d_slice) * inv_fade_in_distance;
					else if (high_z < selected_3d_slice)
						alpha = 1.0 - ((float)selected_3d_slice - high_z) * inv_fade_in_distance;
					else
						alpha = 1;

					if (min_distance2 > max_distance2)
						cr->set_source_rgba(0.5, 0.5, 0, alpha);
					else
						cr->set_source_rgba(1, 0, 0, alpha);

					cr->move_to(node.pos[0], node.pos[1]);
					cr->line_to(proj_min_node2[0]+filterHandler.transformation_data.w, proj_min_node2[1]);

					cr->stroke();
					cr->set_line_width(1);
				}
			}

		}
			break;	// case 6
	}

	return true;
}

/**
 * draw the graph 'graph' with the color 'color' to the context cr
 * displace the nodes by (disp_x, disp_y)
 * if matrix is specified, transform the graph nodes with the matrix before drawing
 */
template<typename T>
void CWindowMain<T>::draw_graph(	CGraph<3,T> &graph,
	       				float color[4],
				       	Cairo::RefPtr<Cairo::Context>& cr,
				       	float disp_x = 0, float disp_y = 0,
					CMatrix4<T> *matrix = NULL
)
{
	float colorRotation[12][4] = 
	{
		{ 1, 0, 0, 1 },
		{ 0, 1, 0, 1 },
		{ 0, 0, 1, 1 },
		{ 1, 1, 0, 1 },
		{ 0, 1, 1, 1 },
		{ 1, 0, 1, 1 }
	};
	float *draw_color = color;

	if (color[0] >= 0)
		cr->set_source_rgba(color[0], color[1], color[2], color[3]);

	#define COLOR_ROTATION_SIZE (sizeof(colorRotation)/sizeof(float)/3)

	int strip_nr = 0;
	for (typename list<typename CGraph<3,T>::CEdge>::iterator edges_i = graph.edges.begin(); edges_i != graph.edges.end(); edges_i++)
	{
		list<typename CGraph<3,T>::CNode> *strip = &(*edges_i).nodes;
		if (!(*strip).empty())
		{
			// setup color
			if (color[0] < 0)
			{
				draw_color = colorRotation[strip_nr % COLOR_ROTATION_SIZE];

				// set color
				cr->set_source_rgba(draw_color[0], draw_color[1], draw_color[2], draw_color[3]);
			}

			// if there's just one node in the strip, even draw this one!
			if ((*strip).size() == 1)
			{
				// load first point
				CVector<3,T> pos = (*((*strip).begin())).pos;
				if (matrix != NULL)	pos = (*matrix)*pos;

				if ((int)(pos)[2] != selected_3d_slice)
					continue;

				int p[2] = {(int)pos[0], (int)pos[1]};
				cr->set_line_width(4);
				cr->move_to(p[0]-1 + disp_x, p[1]-1 + disp_y);
				cr->line_to(p[0]+1 + disp_x, p[1]+1 + disp_y);
				cr->stroke();
				cr->set_line_width(1);
				continue;
			}

			// load iterator for list and the first point
			typename list<typename CGraph<3,T>::CNode>::iterator i = (*strip).begin();

			CVector<3,T> prev_pos = (*i).pos;
			if (matrix != NULL)	prev_pos = (*matrix)*prev_pos;

			i++;
			for (; i != (*strip).end(); i++)
			{
				int prev_point_00 = (int)prev_pos[0];
				int prev_point_01 = (int)prev_pos[1];

				cr->move_to(prev_point_00 + disp_x, prev_point_01 + disp_y);

				T z1 = prev_pos[2];

				prev_pos = (*i).pos;
				if (matrix != NULL)	prev_pos = (*matrix)*prev_pos;

				T z2 = prev_pos[2];

				int prev_point_10 = (int)prev_pos[0];
				int prev_point_11 = (int)prev_pos[1];

				if (z1 < z2)
				{
					if (z1 > selected_3d_slice || z2 < selected_3d_slice)
						continue;
				}
				else
				{
					if (z1 < selected_3d_slice || z2 > selected_3d_slice)
						continue;
				}

				// if both positions in plane are the same, increment one value to draw at least a point
				if (prev_point_00 == prev_point_10 && prev_point_01 == prev_point_11)
				{
					prev_point_10++;
					prev_point_11++;
				}

				cr->line_to(prev_point_10 + disp_x, prev_point_11 + disp_y);
			}
		}

		// draw the strip!
		cr->stroke();

		strip_nr++;
	}
}


/*********************************************************************************************************************
 *********************************************************************************************************************
 * helper functions to set gui parameters loaded from parameter file
 *********************************************************************************************************************
 *********************************************************************************************************************
 */


template<typename T>
void CWindowMain<T>::set_gui_parameter(	Gtk::SpinButton	*spin_button,
					typename CFilterHandler<T>::SParam *tparams,
					const char *button_name,
					string &missing_params
				)
{
		string button_string(button_name);

		while (tparams->name != NULL)
		{
			if (tparams->name == button_string)
			{
				if (isnan(tparams->float_value))
					break;

				switch(tparams->type)
				{
					case CFilterHandler<T>::FLOAT:
						spin_button->set_value(tparams->float_value);	break;
					case CFilterHandler<T>::INT:	spin_button->set_value(tparams->int_value);	break;

					default:
						missing_params += string() + "[WRONG VALUE: " + button_name + "]";
						break;
				}
				return;
			}
			tparams++;
		}

		missing_params += string() + "[" + button_name + "]";
}



template<typename T>
void CWindowMain<T>::set_gui_parameter(	Gtk::CheckButton	*check_button,
					typename CFilterHandler<T>::SParam *tparams,
					const char *button_name,
					string &missing_params
				)
{
		string button_string(button_name);

		while (tparams->name != NULL)
		{
			if (tparams->name == button_string)
			{
				if (isnan(tparams->float_value))
					break;

				switch(tparams->type)
				{
					case CFilterHandler<T>::BOOL:
						check_button->set_active(tparams->bool_value);
						break;

					default:
						missing_params += string() + "[WRONG VALUE: " + button_name + "]";
						break;

				}
				return;
			}
			tparams++;
		}

		missing_params += string() + "[" + button_name + "]";
}




template<typename T>
void CWindowMain<T>::set_gui_parameter(	Gtk::Entry	*entry,
					typename CFilterHandler<T>::SParam *tparams,
					const char *button_name,
					string &missing_params
				)
{
		string button_string(button_name);

		while (tparams->name != NULL)
		{
			if (tparams->name == button_string)
			{
				if (isnan(tparams->float_value))
					break;

				switch(tparams->type)
				{
					case CFilterHandler<T>::FLOAT:
					case CFilterHandler<T>::INT:
					case CFilterHandler<T>::BOOL:
						missing_params += string() + "[WRONG VALUE: " + button_name + "]";
						break;

					case CFilterHandler<T>::STRING:
						entry->set_text(tparams->string_value);
						break;
				}
				return;
			}
			tparams++;
		}

		missing_params += string() + "[" + button_name + "]";
}



template<typename T>
void CWindowMain<T>::on_toolbutton_save()
{
	if (filterHandler.raw_filename.size() == 0)
	{
		statusbar->push("ERROR: Load image data to store the parameters");
		return;
	}

	// store parameters, too
	if (checkbutton_save_with_parameters->get_active())
	{
		typename CFilterHandler<T>::SParam params[] =
		{
			{"spinbutton_raw_extra_window_padding",		CFilterHandler<T>::FLOAT,	spinbutton_raw_extra_window_padding->get_value()},

			{"spinbutton_sphere_min_value",			CFilterHandler<T>::FLOAT,	spinbutton_sphere_min_value->get_value()	},
			{"spinbutton_sphere_max_value",			CFilterHandler<T>::FLOAT,	spinbutton_sphere_max_value->get_value()	},
			{"spinbutton_sphere_min_radius",		CFilterHandler<T>::FLOAT,	spinbutton_sphere_min_radius->get_value()	},
			{"spinbutton_sphere_valid_error",		CFilterHandler<T>::FLOAT,	spinbutton_sphere_valid_error->get_value()	},
			{"spinbutton_sphere_fft_threads",		CFilterHandler<T>::FLOAT,	spinbutton_sphere_fft_threads->get_value()	},

			{"spinbutton_particles_init_distance",		CFilterHandler<T>::FLOAT,	spinbutton_particles_init_distance->get_value()		},
			{"spinbutton_particles_stop_gradient",		CFilterHandler<T>::FLOAT,	spinbutton_particles_stop_gradient->get_value()		},
			{"spinbutton_particles_start_steps",		CFilterHandler<T>::FLOAT,	spinbutton_particles_start_steps->get_value()		},
			{"spinbutton_particles_min_start_radius",	CFilterHandler<T>::FLOAT,	spinbutton_particles_min_start_radius->get_value()	},

			{"spinbutton_graph_min_next_distance",		CFilterHandler<T>::FLOAT,	spinbutton_graph_min_next_distance->get_value()		},
			{"spinbutton_graph_max_next_distance",		CFilterHandler<T>::FLOAT,	spinbutton_graph_max_next_distance->get_value()		},
			{"spinbutton_graph_ignore_start_border_size",	CFilterHandler<T>::FLOAT,	spinbutton_graph_ignore_start_border_size->get_value()	},
			{"spinbutton_graph_connect_strip_distance_scale",	CFilterHandler<T>::FLOAT,	spinbutton_graph_connect_strip_distance_scale->get_value()	},

			{"spinbutton_matching_simple_unprojected_distance",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_unprojected_distance->get_value(), 0	},
			{"spinbutton_matching_simple_min_matrix_generation_point_distance",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_min_matrix_generation_point_distance->get_value(), 0	},
			{"checkbutton_matching_mutual_graph_distance",	CFilterHandler<T>::BOOL,	0, 0, string(), checkbutton_matching_mutual_graph_distance->get_active()	},
			{"spinbutton_matching_simple_adaptive_max_points",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_adaptive_max_points->get_value(), 0	},
			{"spinbutton_matching_simple_adaptive_start_min_distance",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_adaptive_start_min_distance->get_value(), 0	},
			{"spinbutton_matching_simple_adaptive_start_min_radius",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_adaptive_start_min_radius->get_value(), 0	},
			{"checkbutton_matching_simple_use_point_to_line_distance",	CFilterHandler<T>::BOOL,	0, 0, string(), checkbutton_matching_simple_use_point_to_line_distance->get_active()	},

			{"spinbutton_matching_simple_max_rotation_angle",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_max_rotation_angle->get_value(), 0		},
			{"spinbutton_matching_simple_min_scale_factor",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_min_scale_factor->get_value(), 0		},
			{"spinbutton_matching_simple_max_scale_factor",		CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_max_scale_factor->get_value(), 0		},
			{"spinbutton_matching_simple_min_shear_factor",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_min_shear_factor->get_value(), 0		},
			{"spinbutton_matching_simple_max_shear_factor",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_max_shear_factor->get_value(), 0		},
			{"spinbutton_matching_simple_max_translation_dist_x",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_max_translation_dist_x->get_value(), 0		},
			{"spinbutton_matching_simple_max_translation_dist_y",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_max_translation_dist_y->get_value(), 0		},
			{"spinbutton_matching_simple_max_translation_dist_z",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_max_translation_dist_z->get_value(), 0		},
			{"spinbutton_matching_simple_pre_translation_x",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_pre_translation_x->get_value(), 0		},
			{"spinbutton_matching_simple_pre_translation_y",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_pre_translation_y->get_value(), 0		},
			{"spinbutton_matching_simple_pre_translation_z",	CFilterHandler<T>::FLOAT,	spinbutton_matching_simple_pre_translation_z->get_value(), 0		},

			{"entry_reference_graph_file",	CFilterHandler<T>::STRING,	0, 0, entry_reference_graph_file->get_text()	},

			{"spinbutton_matching_matrix_00",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[0][0]->get_value(), 0		},
			{"spinbutton_matching_matrix_01",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[0][1]->get_value(), 0		},
			{"spinbutton_matching_matrix_02",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[0][2]->get_value(), 0		},
			{"spinbutton_matching_matrix_03",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[0][3]->get_value(), 0		},

			{"spinbutton_matching_matrix_10",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[1][0]->get_value(), 0		},
			{"spinbutton_matching_matrix_11",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[1][1]->get_value(), 0		},
			{"spinbutton_matching_matrix_12",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[1][2]->get_value(), 0		},
			{"spinbutton_matching_matrix_13",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[1][3]->get_value(), 0		},

			{"spinbutton_matching_matrix_20",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[2][0]->get_value(), 0		},
			{"spinbutton_matching_matrix_21",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[2][1]->get_value(), 0		},
			{"spinbutton_matching_matrix_22",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[2][2]->get_value(), 0		},
			{"spinbutton_matching_matrix_23",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[2][3]->get_value(), 0		},

			{"spinbutton_matching_matrix_30",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[3][0]->get_value(), 0		},
			{"spinbutton_matching_matrix_31",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[3][1]->get_value(), 0		},
			{"spinbutton_matching_matrix_32",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[3][2]->get_value(), 0		},
			{"spinbutton_matching_matrix_33",	CFilterHandler<T>::FLOAT,	spinbutton_matching_matrix[3][3]->get_value(), 0		},

			{"entry_transformation_reference_data_file",	CFilterHandler<T>::STRING,	0, 0, entry_transformation_reference_data_file->get_text()	},

			{NULL, CFilterHandler<T>::FLOAT}
		};

		filterHandler.save(params);
	}
	else
	{
		filterHandler.save(NULL);
	}

	statusbar->push("Image and data stored");
}



template<typename T>
void CWindowMain<T>::on_toolbutton_open()
{
	if (!load_raw_data_with_dialog())
		return;

	// load parameters, too
	if (checkbutton_open_with_parameters->get_active())
	{
		float nan = std::numeric_limits<float>::quiet_NaN();

		// available parameters
		typename CFilterHandler<T>::SParam params[] =
		{
			{"spinbutton_raw_extra_window_padding",		CFilterHandler<T>::FLOAT,	nan},

			{"spinbutton_sphere_min_value",			CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_sphere_max_value",			CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_sphere_min_radius",		CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_sphere_valid_error",		CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_sphere_fft_threads",		CFilterHandler<T>::FLOAT,	nan},

			{"spinbutton_particles_init_distance",		CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_particles_stop_gradient",		CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_particles_start_steps",		CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_particles_min_start_radius",	CFilterHandler<T>::FLOAT,	nan},

			{"spinbutton_graph_min_next_distance",		CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_graph_max_next_distance",		CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_graph_ignore_start_border_size",	CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_graph_connect_strip_distance_scale",	CFilterHandler<T>::FLOAT,	nan},

			// matching simple
			{"spinbutton_matching_simple_unprojected_distance",	CFilterHandler<T>::FLOAT,	nan	},
			{"spinbutton_matching_simple_min_matrix_generation_point_distance",	CFilterHandler<T>::FLOAT,	nan	},
			{"checkbutton_matching_mutual_graph_distance",	CFilterHandler<T>::BOOL,	nan	},
			{"spinbutton_matching_simple_adaptive_max_points",	CFilterHandler<T>::FLOAT,	nan	},
			{"spinbutton_matching_simple_adaptive_start_min_distance",	CFilterHandler<T>::FLOAT,	nan	},
			{"spinbutton_matching_simple_adaptive_start_min_radius",	CFilterHandler<T>::FLOAT,	nan	},
			{"checkbutton_matching_simple_use_point_to_line_distance",	CFilterHandler<T>::BOOL,	nan	},

			{"spinbutton_matching_simple_max_rotation_angle",	CFilterHandler<T>::FLOAT,	nan	},
			{"spinbutton_matching_simple_min_scale_factor",		CFilterHandler<T>::FLOAT,	nan	},
			{"spinbutton_matching_simple_max_scale_factor",		CFilterHandler<T>::FLOAT,	nan	},
			{"spinbutton_matching_simple_min_shear_factor",		CFilterHandler<T>::FLOAT,	nan	},
			{"spinbutton_matching_simple_max_shear_factor",		CFilterHandler<T>::FLOAT,	nan	},
			{"spinbutton_matching_simple_max_translation_dist_x",		CFilterHandler<T>::FLOAT,	nan	},
			{"spinbutton_matching_simple_max_translation_dist_y",		CFilterHandler<T>::FLOAT,	nan	},
			{"spinbutton_matching_simple_max_translation_dist_z",		CFilterHandler<T>::FLOAT,	nan	},
			{"spinbutton_matching_simple_pre_translation_x",		CFilterHandler<T>::FLOAT,	nan	},
			{"spinbutton_matching_simple_pre_translation_y",		CFilterHandler<T>::FLOAT,	nan	},
			{"spinbutton_matching_simple_pre_translation_z",		CFilterHandler<T>::FLOAT,	nan	},

			{"entry_reference_graph_file",	CFilterHandler<T>::STRING,	nan},

			{"spinbutton_matching_matrix_00",	CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_matching_matrix_01",	CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_matching_matrix_02",	CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_matching_matrix_03",	CFilterHandler<T>::FLOAT,	nan},

			{"spinbutton_matching_matrix_10",	CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_matching_matrix_11",	CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_matching_matrix_12",	CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_matching_matrix_13",	CFilterHandler<T>::FLOAT,	nan},

			{"spinbutton_matching_matrix_20",	CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_matching_matrix_21",	CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_matching_matrix_22",	CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_matching_matrix_23",	CFilterHandler<T>::FLOAT,	nan},

			{"spinbutton_matching_matrix_30",	CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_matching_matrix_31",	CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_matching_matrix_32",	CFilterHandler<T>::FLOAT,	nan},
			{"spinbutton_matching_matrix_33",	CFilterHandler<T>::FLOAT,	nan},

			{"entry_transformation_reference_data_file",	CFilterHandler<T>::STRING,	nan},

			{NULL, CFilterHandler<T>::FLOAT	}
		};

		filterHandler.load(params);

		string missing_params;

		set_gui_parameter(spinbutton_raw_extra_window_padding,	params,	"spinbutton_raw_extra_window_padding", missing_params);

		set_gui_parameter(spinbutton_sphere_min_value,		params,	"spinbutton_sphere_min_value", missing_params);
		set_gui_parameter(spinbutton_sphere_max_value,		params,	"spinbutton_sphere_max_value", missing_params);
		set_gui_parameter(spinbutton_sphere_min_radius,		params,	"spinbutton_sphere_min_radius", missing_params);
		set_gui_parameter(spinbutton_sphere_valid_error,	params,	"spinbutton_sphere_valid_error", missing_params);
		set_gui_parameter(spinbutton_sphere_fft_threads,	params,	"spinbutton_sphere_fft_threads", missing_params);

		set_gui_parameter(spinbutton_particles_init_distance,	params,	"spinbutton_particles_init_distance", missing_params);
		set_gui_parameter(spinbutton_particles_stop_gradient,	params,	"spinbutton_particles_stop_gradient", missing_params);
		set_gui_parameter(spinbutton_particles_start_steps,	params,	"spinbutton_particles_start_steps", missing_params);
		set_gui_parameter(spinbutton_particles_min_start_radius,	params,	"spinbutton_particles_min_start_radius", missing_params);

		set_gui_parameter(spinbutton_graph_min_next_distance,		params,	"spinbutton_graph_min_next_distance", missing_params);
		set_gui_parameter(spinbutton_graph_max_next_distance,		params,	"spinbutton_graph_max_next_distance", missing_params);
		set_gui_parameter(spinbutton_graph_ignore_start_border_size,	params,	"spinbutton_graph_ignore_start_border_size", missing_params);
		set_gui_parameter(spinbutton_graph_connect_strip_distance_scale,	params,	"spinbutton_graph_connect_strip_distance_scale", missing_params);

		set_gui_parameter(spinbutton_matching_simple_unprojected_distance,	params,	"spinbutton_matching_simple_unprojected_distance", missing_params);
		set_gui_parameter(spinbutton_matching_simple_min_matrix_generation_point_distance,	params,	"spinbutton_matching_simple_min_matrix_generation_point_distance", missing_params);
		set_gui_parameter(checkbutton_matching_mutual_graph_distance,	params,	"checkbutton_matching_mutual_graph_distance", missing_params);
		set_gui_parameter(spinbutton_matching_simple_adaptive_max_points,	params,	"spinbutton_matching_simple_adaptive_max_points", missing_params);
		set_gui_parameter(spinbutton_matching_simple_adaptive_start_min_distance,	params,	"spinbutton_matching_simple_adaptive_start_min_distance", missing_params);
		set_gui_parameter(spinbutton_matching_simple_adaptive_start_min_radius,	params,	"spinbutton_matching_simple_adaptive_start_min_radius", missing_params);
		set_gui_parameter(checkbutton_matching_simple_use_point_to_line_distance,	params,	"checkbutton_matching_simple_use_point_to_line_distance", missing_params);

		set_gui_parameter(spinbutton_matching_simple_max_rotation_angle,	params,	"spinbutton_matching_simple_max_rotation_angle", missing_params);
		set_gui_parameter(spinbutton_matching_simple_min_scale_factor,	params,	"spinbutton_matching_simple_min_scale_factor", missing_params);
		set_gui_parameter(spinbutton_matching_simple_max_scale_factor,	params,	"spinbutton_matching_simple_max_scale_factor", missing_params);
		set_gui_parameter(spinbutton_matching_simple_min_shear_factor,	params,	"spinbutton_matching_simple_min_shear_factor", missing_params);
		set_gui_parameter(spinbutton_matching_simple_max_shear_factor,	params,	"spinbutton_matching_simple_max_shear_factor", missing_params);
		set_gui_parameter(spinbutton_matching_simple_max_translation_dist_x,	params,	"spinbutton_matching_simple_max_translation_dist_x", missing_params);
		set_gui_parameter(spinbutton_matching_simple_max_translation_dist_y,	params,	"spinbutton_matching_simple_max_translation_dist_y", missing_params);
		set_gui_parameter(spinbutton_matching_simple_max_translation_dist_z,	params,	"spinbutton_matching_simple_max_translation_dist_z", missing_params);
		set_gui_parameter(spinbutton_matching_simple_pre_translation_x,	params,	"spinbutton_matching_simple_pre_translation_x", missing_params);
		set_gui_parameter(spinbutton_matching_simple_pre_translation_y,	params,	"spinbutton_matching_simple_pre_translation_y", missing_params);
		set_gui_parameter(spinbutton_matching_simple_pre_translation_z,	params,	"spinbutton_matching_simple_pre_translation_z", missing_params);

		set_gui_parameter(entry_reference_graph_file,	params,	"entry_reference_graph_file", missing_params);

		set_gui_parameter(spinbutton_matching_matrix[0][0],	params,	"spinbutton_matching_matrix_00", missing_params);
		set_gui_parameter(spinbutton_matching_matrix[0][1],	params,	"spinbutton_matching_matrix_01", missing_params);
		set_gui_parameter(spinbutton_matching_matrix[0][2],	params,	"spinbutton_matching_matrix_02", missing_params);
		set_gui_parameter(spinbutton_matching_matrix[0][3],	params,	"spinbutton_matching_matrix_03", missing_params);

		set_gui_parameter(spinbutton_matching_matrix[1][0],	params,	"spinbutton_matching_matrix_10", missing_params);
		set_gui_parameter(spinbutton_matching_matrix[1][1],	params,	"spinbutton_matching_matrix_11", missing_params);
		set_gui_parameter(spinbutton_matching_matrix[1][2],	params,	"spinbutton_matching_matrix_12", missing_params);
		set_gui_parameter(spinbutton_matching_matrix[1][3],	params,	"spinbutton_matching_matrix_13", missing_params);

		set_gui_parameter(spinbutton_matching_matrix[2][0],	params,	"spinbutton_matching_matrix_20", missing_params);
		set_gui_parameter(spinbutton_matching_matrix[2][1],	params,	"spinbutton_matching_matrix_21", missing_params);
		set_gui_parameter(spinbutton_matching_matrix[2][2],	params,	"spinbutton_matching_matrix_22", missing_params);
		set_gui_parameter(spinbutton_matching_matrix[2][3],	params,	"spinbutton_matching_matrix_23", missing_params);

		set_gui_parameter(spinbutton_matching_matrix[3][0],	params,	"spinbutton_matching_matrix_30", missing_params);
		set_gui_parameter(spinbutton_matching_matrix[3][1],	params,	"spinbutton_matching_matrix_31", missing_params);
		set_gui_parameter(spinbutton_matching_matrix[3][2],	params,	"spinbutton_matching_matrix_32", missing_params);
		set_gui_parameter(spinbutton_matching_matrix[3][3],	params,	"spinbutton_matching_matrix_33", missing_params);

		set_gui_parameter(entry_transformation_reference_data_file,	params,	"entry_transformation_reference_data_file", missing_params);

		// try to load matching graph
		on_button_matching_load_graph();
		on_button_transformation_load_reference_data();

		if (missing_params.empty())
			statusbar->push("Image and data loaded");
		else
			statusbar->push(string() + "Image and data loaded | WARNING: missing parameter: '" + missing_params + "'");

		update_hscale_limits();
	}
	else
	{
		filterHandler.load();
		statusbar->push("Image and data loaded");
	}
}

#endif
