/*
 * This file is part of the segmentation and registration toolkit (SphereSegRegTk).
 *
 * SphereSegRegTk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SphereSegRegTk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SphereSegRegTk.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __CFILTERPROJECTION3_HPP
#define __CFILTERPROJECTION3_HPP

#include "CMatrix4.hh"

template< typename T >
class CFilterProjection
{
	CMatrix4<T> proj_matrix;
	CVector<3,T> pre_translation;

public:

	void setParameters(
			CMatrix4<T> &p_proj_matrix
			)
	{
		proj_matrix = p_proj_matrix;
	}

	/**
	 * project the raw data to proj_data by the matrix given with proj_matrix
	 * \proj_matrix	matrix which projects points FROM proj_data TO raw_data!!!
	 */
	void filter(CData<T> &raw_data, CData<T> &proj_data, CVector<3,int> size)
	{
		proj_data.resize(size);

		// first we set all transformation data to zero

		for (int z = 0; z < proj_data.d; z++)
		for (int y = 0; y < proj_data.h; y++)
		for (int x = 0; x < proj_data.w; x++)
		proj_data.set(x, y, z, (T)1000000);

		// now we reproject the values from raw_data by applying the matrix
		CVector<3,T> trans_pos;

		for (trans_pos[2] = 0; trans_pos[2] < size[2]; trans_pos[2]++)
		for (trans_pos[1] = 0; trans_pos[1] < size[1]; trans_pos[1]++)
		for (trans_pos[0] = 0; trans_pos[0] < size[0]; trans_pos[0]++)
		{
			CVector<3,T> raw_pos = proj_matrix*trans_pos;

			CVector<3,int> raw_posi(raw_pos);
			if (!raw_data.inCheck(raw_posi))
				continue;

			// TODO: use interpolation to get value
			T value = raw_data.get(raw_posi);

			CVector<3,int> trans_posi(trans_pos);
			proj_data.set(trans_posi, value);
		}
	}

};

#endif
